@@ -29,48 +29,48 @@
 #define R_API_BEGIN() \
   try {
 #define R_API_END() } \
-  catch(std::exception& ex) { LGBM_SetLastError(ex.what()); return R_NilValue;} \
-  catch(std::string& ex) { LGBM_SetLastError(ex.c_str()); return R_NilValue; } \
-  catch(...) { LGBM_SetLastError("unknown exception"); return R_NilValue;} \
-  return R_NilValue;
+  catch(std::exception& ex) { LGBM_SetLastError(ex.what()); } \
+  catch(std::string& ex) { LGBM_SetLastError(ex.c_str()); } \
+  catch(...) { LGBM_SetLastError("unknown exception"); } 
 
 #define CHECK_CALL(x) \
   if ((x) != 0) { \
     Rf_error(LGBM_GetLastError()); \
-    return R_NilValue;\
   }
 
 using LightGBM::Common::Split;
 using LightGBM::Log;
 
-SEXP LGBM_GetLastError_R() {
-	SEXP out;
-	out = PROTECT(Rf_allocVector(STRSXP, 1));
-	SET_STRING_ELT(out, 0, Rf_mkChar(LGBM_GetLastError()));
-	UNPROTECT(1);
-	return out;
-}
-
 SEXP LGBM_HandleIsNull_R(SEXP handle) {
 	return Rf_ScalarLogical(R_ExternalPtrAddr(handle) == NULL);
 }
 
+void _DatasetFinalizer(SEXP handle) {
+	LGBM_DatasetFree_R(handle);
+}
+
 SEXP LGBM_DatasetCreateFromFile_R(SEXP filename,
 	SEXP parameters,
 	SEXP reference) {
+	int n_protected = 0;
 	SEXP ret;
-	R_API_BEGIN();
 	DatasetHandle handle = nullptr;
 	DatasetHandle ref = nullptr;
 	if (!Rf_isNull(reference)) {
 		ref = R_ExternalPtrAddr(reference);
 	}
-	CHECK_CALL(LGBM_DatasetCreateFromFile(CHAR(Rf_asChar(filename)), CHAR(Rf_asChar(parameters)),
-		ref, &handle));
-	ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
-	UNPROTECT(1);
-	return ret;
+	const char* filename_ptr = CHAR(PROTECT(Rf_asChar(filename)));
+	n_protected++;
+	const char* parameters_ptr = CHAR(PROTECT(Rf_asChar(parameters)));
+	n_protected++;
+	R_API_BEGIN();
+	CHECK_CALL(LGBM_DatasetCreateFromFile(filename_ptr, parameters_ptr, ref, &handle));
 	R_API_END();
+	ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
+	n_protected++;
+	R_RegisterCFinalizerEx(ret, _DatasetFinalizer, TRUE);
+	UNPROTECT(n_protected);
+	return ret;
 }
 
 SEXP LGBM_DatasetCreateFromCSC_R(SEXP indptr,
@@ -81,27 +81,31 @@ SEXP LGBM_DatasetCreateFromCSC_R(SEXP indptr,
 	SEXP num_row,
 	SEXP parameters,
 	SEXP reference) {
+	int n_protected = 0;
 	SEXP ret;
-	R_API_BEGIN();
 	const int* p_indptr = INTEGER(indptr);
 	const int* p_indices = INTEGER(indices);
 	const double* p_data = REAL(data);
-
 	int64_t nindptr = static_cast<int64_t>(Rf_asInteger(num_indptr));
 	int64_t ndata = static_cast<int64_t>(Rf_asInteger(nelem));
 	int64_t nrow = static_cast<int64_t>(Rf_asInteger(num_row));
+	const char* parameters_ptr = CHAR(PROTECT(Rf_asChar(parameters)));
+	n_protected++;
 	DatasetHandle handle = nullptr;
 	DatasetHandle ref = nullptr;
 	if (!Rf_isNull(reference)) {
 		ref = R_ExternalPtrAddr(reference);
 	}
+	R_API_BEGIN();
 	CHECK_CALL(LGBM_DatasetCreateFromCSC(p_indptr, C_API_DTYPE_INT32, p_indices,
 		p_data, C_API_DTYPE_FLOAT64, nindptr, ndata,
-		nrow, CHAR(Rf_asChar(parameters)), ref, &handle));
-	ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
-	UNPROTECT(1);
-	return ret;
+		nrow, parameters_ptr, ref, &handle));
 	R_API_END();
+	ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
+	n_protected++;
+	R_RegisterCFinalizerEx(ret, _DatasetFinalizer, TRUE);
+	UNPROTECT(n_protected);
+	return ret;
 }
 
 SEXP LGBM_DatasetCreateFromMat_R(SEXP data,
@@ -109,30 +113,35 @@ SEXP LGBM_DatasetCreateFromMat_R(SEXP data,
 	SEXP num_col,
 	SEXP parameters,
 	SEXP reference) {
+	int n_protected = 0;
 	SEXP ret;
-	R_API_BEGIN();
 	int32_t nrow = static_cast<int32_t>(Rf_asInteger(num_row));
 	int32_t ncol = static_cast<int32_t>(Rf_asInteger(num_col));
 	double* p_mat = REAL(data);
+	const char* parameters_ptr = CHAR(PROTECT(Rf_asChar(parameters)));
+	n_protected++;
 	DatasetHandle handle = nullptr;
 	DatasetHandle ref = nullptr;
 	if (!Rf_isNull(reference)) {
 		ref = R_ExternalPtrAddr(reference);
 	}
+	R_API_BEGIN();
 	CHECK_CALL(LGBM_DatasetCreateFromMat(p_mat, C_API_DTYPE_FLOAT64, nrow, ncol, COL_MAJOR,
-		CHAR(Rf_asChar(parameters)), ref, &handle));
-	ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
-	UNPROTECT(1);
-	return ret;
+		parameters_ptr, ref, &handle));
 	R_API_END();
+	ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
+	n_protected++;
+	R_RegisterCFinalizerEx(ret, _DatasetFinalizer, TRUE);
+	UNPROTECT(n_protected);
+	return ret;
 }
 
 SEXP LGBM_DatasetGetSubset_R(SEXP handle,
 	SEXP used_row_indices,
 	SEXP len_used_row_indices,
 	SEXP parameters) {
+	int n_protected = 0;
 	SEXP ret;
-	R_API_BEGIN();
 	int32_t len = static_cast<int32_t>(Rf_asInteger(len_used_row_indices));
 	std::vector<int32_t> idxvec(len);
 	// convert from one-based to  zero-based index
@@ -140,35 +149,43 @@ SEXP LGBM_DatasetGetSubset_R(SEXP handle,
 	for (int32_t i = 0; i < len; ++i) {
 		idxvec[i] = static_cast<int32_t>(INTEGER(used_row_indices)[i] - 1);
 	}
+	const char* parameters_ptr = CHAR(PROTECT(Rf_asChar(parameters)));
+	n_protected++;
 	DatasetHandle res = nullptr;
+	R_API_BEGIN();
 	CHECK_CALL(LGBM_DatasetGetSubset(R_ExternalPtrAddr(handle),
-		idxvec.data(), len, CHAR(Rf_asChar(parameters)),
+		idxvec.data(), len, parameters_ptr,
 		&res));
-	ret = PROTECT(R_MakeExternalPtr(res, R_NilValue, R_NilValue));
-	UNPROTECT(1);
-	return ret;
 	R_API_END();
+	ret = PROTECT(R_MakeExternalPtr(res, R_NilValue, R_NilValue));
+	n_protected++;
+	R_RegisterCFinalizerEx(ret, _DatasetFinalizer, TRUE);
+	UNPROTECT(n_protected);
+	return ret;
 }
 
 SEXP LGBM_DatasetSetFeatureNames_R(SEXP handle,
 	SEXP feature_names) {
-	R_API_BEGIN();
-	auto vec_names = Split(CHAR(Rf_asChar(feature_names)), '\t');
+	auto vec_names = Split(CHAR(PROTECT(Rf_asChar(feature_names))), '\t');
 	std::vector<const char*> vec_sptr;
 	int len = static_cast<int>(vec_names.size());
 	for (int i = 0; i < len; ++i) {
 		vec_sptr.push_back(vec_names[i].c_str());
 	}
+	R_API_BEGIN();
 	CHECK_CALL(LGBM_DatasetSetFeatureNames(R_ExternalPtrAddr(handle),
 		vec_sptr.data(), len));
 	R_API_END();
+	UNPROTECT(1);
+	return R_NilValue;
 }
 
 SEXP LGBM_DatasetGetFeatureNames_R(SEXP handle) {
 	SEXP feature_names;
-	R_API_BEGIN();
 	int len = 0;
+	R_API_BEGIN();
 	CHECK_CALL(LGBM_DatasetGetNumFeature(R_ExternalPtrAddr(handle), &len));
+	R_API_END();
 	const size_t reserved_string_size = 256;
 	std::vector<std::vector<char>> names(len);
 	std::vector<char*> ptr_names(len);
@@ -178,12 +195,14 @@ SEXP LGBM_DatasetGetFeatureNames_R(SEXP handle) {
 	}
 	int out_len;
 	size_t required_string_size;
+	R_API_BEGIN();
 	CHECK_CALL(
 		LGBM_DatasetGetFeatureNames(
 			R_ExternalPtrAddr(handle),
 			len, &out_len,
 			reserved_string_size, &required_string_size,
 			ptr_names.data()));
+	R_API_END();
 	// if any feature names were larger than allocated size,
 	// allow for a larger size and try again
 	if (required_string_size > reserved_string_size) {
@@ -191,6 +210,7 @@ SEXP LGBM_DatasetGetFeatureNames_R(SEXP handle) {
 			names[i].resize(required_string_size);
 			ptr_names[i] = names[i].data();
 		}
+		R_API_BEGIN();
 		CHECK_CALL(
 			LGBM_DatasetGetFeatureNames(
 				R_ExternalPtrAddr(handle),
@@ -199,6 +219,7 @@ SEXP LGBM_DatasetGetFeatureNames_R(SEXP handle) {
 				required_string_size,
 				&required_string_size,
 				ptr_names.data()));
+		R_API_END();
 	}
 	CHECK_EQ(len, out_len);
 	feature_names = PROTECT(Rf_allocVector(STRSXP, len));
@@ -207,33 +228,36 @@ SEXP LGBM_DatasetGetFeatureNames_R(SEXP handle) {
 	}
 	UNPROTECT(1);
 	return feature_names;
-	R_API_END();
 }
 
 SEXP LGBM_DatasetSaveBinary_R(SEXP handle,
 	SEXP filename) {
+	const char* filename_ptr = CHAR(PROTECT(Rf_asChar(filename)));
 	R_API_BEGIN();
 	CHECK_CALL(LGBM_DatasetSaveBinary(R_ExternalPtrAddr(handle),
-		CHAR(Rf_asChar(filename))));
+		filename_ptr));
 	R_API_END();
+	UNPROTECT(1);
+	return R_NilValue;
 }
 
 SEXP LGBM_DatasetFree_R(SEXP handle) {
 	R_API_BEGIN();
-	if (R_ExternalPtrAddr(handle) != nullptr) {
+	if (!Rf_isNull(handle) && R_ExternalPtrAddr(handle)) {
 		CHECK_CALL(LGBM_DatasetFree(R_ExternalPtrAddr(handle)));
 		R_ClearExternalPtr(handle);
 	}
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP LGBM_DatasetSetField_R(SEXP handle,
 	SEXP field_name,
 	SEXP field_data,
 	SEXP num_element) {
-	R_API_BEGIN();
 	int len = Rf_asInteger(num_element);
-	const char* name = CHAR(Rf_asChar(field_name));
+	const char* name = CHAR(PROTECT(Rf_asChar(field_name)));
+	R_API_BEGIN();
 	if (!strcmp("group", name) || !strcmp("query", name)) {
 		std::vector<int32_t> vec(len);
 #pragma omp parallel for schedule(static, 512) if (len >= 1024)
@@ -254,18 +278,19 @@ SEXP LGBM_DatasetSetField_R(SEXP handle,
 		CHECK_CALL(LGBM_DatasetSetField(R_ExternalPtrAddr(handle), name, vec.data(), len, C_API_DTYPE_FLOAT32));
 	}
 	R_API_END();
+	UNPROTECT(1);
+	return R_NilValue;
 }
 
 SEXP LGBM_DatasetGetField_R(SEXP handle,
 	SEXP field_name,
 	SEXP field_data) {
-	R_API_BEGIN();
-	const char* name = CHAR(Rf_asChar(field_name));
+	const char* name = CHAR(PROTECT(Rf_asChar(field_name)));
 	int out_len = 0;
 	int out_type = 0;
 	const void* res;
+	R_API_BEGIN();
 	CHECK_CALL(LGBM_DatasetGetField(R_ExternalPtrAddr(handle), name, &out_len, &res, &out_type));
-
 	if (!strcmp("group", name) || !strcmp("query", name)) {
 		auto p_data = reinterpret_cast<const int32_t*>(res);
 		// convert from boundaries to size
@@ -289,29 +314,37 @@ SEXP LGBM_DatasetGetField_R(SEXP handle,
 		}
 	}
 	R_API_END();
+	UNPROTECT(1);
+	return R_NilValue;
 }
 
 SEXP LGBM_DatasetGetFieldSize_R(SEXP handle,
 	SEXP field_name,
 	SEXP out) {
-	R_API_BEGIN();
-	const char* name = CHAR(Rf_asChar(field_name));
+	const char* name = CHAR(PROTECT(Rf_asChar(field_name)));
 	int out_len = 0;
 	int out_type = 0;
 	const void* res;
+	R_API_BEGIN();
 	CHECK_CALL(LGBM_DatasetGetField(R_ExternalPtrAddr(handle), name, &out_len, &res, &out_type));
 	if (!strcmp("group", name) || !strcmp("query", name)) {
 		out_len -= 1;
 	}
 	INTEGER(out)[0] = out_len;
 	R_API_END();
+	UNPROTECT(1);
+	return R_NilValue;
 }
 
 SEXP LGBM_DatasetUpdateParamChecking_R(SEXP old_params,
 	SEXP new_params) {
+	const char* old_params_ptr = CHAR(PROTECT(Rf_asChar(old_params)));
+	const char* new_params_ptr = CHAR(PROTECT(Rf_asChar(new_params)));
 	R_API_BEGIN();
-	CHECK_CALL(LGBM_DatasetUpdateParamChecking(CHAR(Rf_asChar(old_params)), CHAR(Rf_asChar(new_params))));
+	CHECK_CALL(LGBM_DatasetUpdateParamChecking(old_params_ptr, new_params_ptr));
 	R_API_END();
+	UNPROTECT(2);
+	return R_NilValue;
 }
 
 SEXP LGBM_DatasetGetNumData_R(SEXP handle, SEXP out) {
@@ -320,6 +353,7 @@ SEXP LGBM_DatasetGetNumData_R(SEXP handle, SEXP out) {
 	CHECK_CALL(LGBM_DatasetGetNumData(R_ExternalPtrAddr(handle), &nrow));
 	INTEGER(out)[0] = nrow;
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP LGBM_DatasetGetNumFeature_R(SEXP handle,
@@ -329,66 +363,92 @@ SEXP LGBM_DatasetGetNumFeature_R(SEXP handle,
 	CHECK_CALL(LGBM_DatasetGetNumFeature(R_ExternalPtrAddr(handle), &nfeature));
 	INTEGER(out)[0] = nfeature;
 	R_API_END();
+	return R_NilValue;
 }
 
 // --- start Booster interfaces
 
+void _BoosterFinalizer(SEXP handle) {
+	LGBM_BoosterFree_R(handle);
+}
+
 SEXP LGBM_BoosterFree_R(SEXP handle) {
 	R_API_BEGIN();
-	if (R_ExternalPtrAddr(handle) != nullptr) {
+	if (!Rf_isNull(handle) && R_ExternalPtrAddr(handle)) {
 		CHECK_CALL(LGBM_BoosterFree(R_ExternalPtrAddr(handle)));
 		R_ClearExternalPtr(handle);
 	}
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterCreate_R(SEXP train_data,
 	SEXP parameters) {
+	int n_protected = 0;
 	SEXP ret;
-	R_API_BEGIN();
+	const char* parameters_ptr = CHAR(PROTECT(Rf_asChar(parameters)));
+	n_protected++;
 	BoosterHandle handle = nullptr;
-	CHECK_CALL(LGBM_BoosterCreate(R_ExternalPtrAddr(train_data), CHAR(Rf_asChar(parameters)), &handle));
-	ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
-	UNPROTECT(1);
-	return ret;
+	R_API_BEGIN();
+	CHECK_CALL(LGBM_BoosterCreate(R_ExternalPtrAddr(train_data), parameters_ptr, &handle));
 	R_API_END();
+	ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
+	n_protected++;
+	R_RegisterCFinalizerEx(ret, _BoosterFinalizer, TRUE);
+	UNPROTECT(n_protected);
+	return ret;
 }
 
 SEXP LGBM_GPBoosterCreate_R(SEXP train_data,
 	SEXP parameters,
 	SEXP re_model) {
+	int n_protected = 0;
 	SEXP ret;
-	R_API_BEGIN();
+	const char* parameters_ptr = CHAR(PROTECT(Rf_asChar(parameters)));
+	n_protected++;
 	BoosterHandle handle = nullptr;
-	CHECK_CALL(LGBM_GPBoosterCreate(R_ExternalPtrAddr(train_data), CHAR(Rf_asChar(parameters)), R_ExternalPtrAddr(re_model), &handle));
-	ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
-	UNPROTECT(1);
-	return ret;
+	R_API_BEGIN();
+	CHECK_CALL(LGBM_GPBoosterCreate(R_ExternalPtrAddr(train_data), parameters_ptr, R_ExternalPtrAddr(re_model), &handle));
 	R_API_END();
+	ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
+	n_protected++;
+	R_RegisterCFinalizerEx(ret, _BoosterFinalizer, TRUE);
+	UNPROTECT(n_protected);
+	return ret;
 }
 
 SEXP LGBM_BoosterCreateFromModelfile_R(SEXP filename) {
+	int n_protected = 0;
 	SEXP ret;
-	R_API_BEGIN();
 	int out_num_iterations = 0;
+	const char* filename_ptr = CHAR(PROTECT(Rf_asChar(filename)));
+	n_protected++;
 	BoosterHandle handle = nullptr;
-	CHECK_CALL(LGBM_BoosterCreateFromModelfile(CHAR(Rf_asChar(filename)), &out_num_iterations, &handle));
-	ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
-	UNPROTECT(1);
-	return ret;
+	R_API_BEGIN();
+	CHECK_CALL(LGBM_BoosterCreateFromModelfile(filename_ptr, &out_num_iterations, &handle));
 	R_API_END();
+	ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
+	n_protected++;
+	R_RegisterCFinalizerEx(ret, _BoosterFinalizer, TRUE);
+	UNPROTECT(n_protected);
+	return ret;
 }
 
 SEXP LGBM_BoosterLoadModelFromString_R(SEXP model_str) {
+	int n_protected = 0;
 	SEXP ret;
-	R_API_BEGIN();
 	int out_num_iterations = 0;
+	const char* model_str_ptr = CHAR(PROTECT(Rf_asChar(model_str)));
+	n_protected++;
 	BoosterHandle handle = nullptr;
-	CHECK_CALL(LGBM_BoosterLoadModelFromString(CHAR(Rf_asChar(model_str)), &out_num_iterations, &handle));
-	ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
-	UNPROTECT(1);
-	return ret;
+	R_API_BEGIN();
+	CHECK_CALL(LGBM_BoosterLoadModelFromString(model_str_ptr, &out_num_iterations, &handle));
 	R_API_END();
+	ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
+	n_protected++;
+	R_RegisterCFinalizerEx(ret, _BoosterFinalizer, TRUE);
+	UNPROTECT(n_protected);
+	return ret;
 }
 
 SEXP LGBM_BoosterMerge_R(SEXP handle,
@@ -396,6 +456,7 @@ SEXP LGBM_BoosterMerge_R(SEXP handle,
 	R_API_BEGIN();
 	CHECK_CALL(LGBM_BoosterMerge(R_ExternalPtrAddr(handle), R_ExternalPtrAddr(other_handle)));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterAddValidData_R(SEXP handle,
@@ -403,6 +464,7 @@ SEXP LGBM_BoosterAddValidData_R(SEXP handle,
 	R_API_BEGIN();
 	CHECK_CALL(LGBM_BoosterAddValidData(R_ExternalPtrAddr(handle), R_ExternalPtrAddr(valid_data)));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterResetTrainingData_R(SEXP handle,
@@ -410,13 +472,17 @@ SEXP LGBM_BoosterResetTrainingData_R(SEXP handle,
 	R_API_BEGIN();
 	CHECK_CALL(LGBM_BoosterResetTrainingData(R_ExternalPtrAddr(handle), R_ExternalPtrAddr(train_data)));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterResetParameter_R(SEXP handle,
 	SEXP parameters) {
+	const char* parameters_ptr = CHAR(PROTECT(Rf_asChar(parameters)));
 	R_API_BEGIN();
-	CHECK_CALL(LGBM_BoosterResetParameter(R_ExternalPtrAddr(handle), CHAR(Rf_asChar(parameters))));
+	CHECK_CALL(LGBM_BoosterResetParameter(R_ExternalPtrAddr(handle), parameters_ptr));
 	R_API_END();
+	UNPROTECT(1);
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterGetNumClasses_R(SEXP handle,
@@ -426,6 +492,7 @@ SEXP LGBM_BoosterGetNumClasses_R(SEXP handle,
 	CHECK_CALL(LGBM_BoosterGetNumClasses(R_ExternalPtrAddr(handle), &num_class));
 	INTEGER(out)[0] = num_class;
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterUpdateOneIter_R(SEXP handle) {
@@ -433,6 +500,7 @@ SEXP LGBM_BoosterUpdateOneIter_R(SEXP handle) {
 	R_API_BEGIN();
 	CHECK_CALL(LGBM_BoosterUpdateOneIter(R_ExternalPtrAddr(handle), &is_finished));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterUpdateOneIterCustom_R(SEXP handle,
@@ -450,12 +518,14 @@ SEXP LGBM_BoosterUpdateOneIterCustom_R(SEXP handle,
 	}
 	CHECK_CALL(LGBM_BoosterUpdateOneIterCustom(R_ExternalPtrAddr(handle), tgrad.data(), thess.data(), &is_finished));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterRollbackOneIter_R(SEXP handle) {
 	R_API_BEGIN();
 	CHECK_CALL(LGBM_BoosterRollbackOneIter(R_ExternalPtrAddr(handle)));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterGetCurrentIteration_R(SEXP handle,
@@ -465,6 +535,7 @@ SEXP LGBM_BoosterGetCurrentIteration_R(SEXP handle,
 	CHECK_CALL(LGBM_BoosterGetCurrentIteration(R_ExternalPtrAddr(handle), &out_iteration));
 	INTEGER(out)[0] = out_iteration;
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterGetUpperBoundValue_R(SEXP handle,
@@ -473,6 +544,7 @@ SEXP LGBM_BoosterGetUpperBoundValue_R(SEXP handle,
 	double* ptr_ret = REAL(out_result);
 	CHECK_CALL(LGBM_BoosterGetUpperBoundValue(R_ExternalPtrAddr(handle), ptr_ret));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterGetLowerBoundValue_R(SEXP handle,
@@ -481,14 +553,15 @@ SEXP LGBM_BoosterGetLowerBoundValue_R(SEXP handle,
 	double* ptr_ret = REAL(out_result);
 	CHECK_CALL(LGBM_BoosterGetLowerBoundValue(R_ExternalPtrAddr(handle), ptr_ret));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterGetEvalNames_R(SEXP handle) {
 	SEXP eval_names;
-	R_API_BEGIN();
 	int len;
+	R_API_BEGIN();
 	CHECK_CALL(LGBM_BoosterGetEvalCounts(R_ExternalPtrAddr(handle), &len));
-
+	R_API_END();
 	const size_t reserved_string_size = 128;
 	std::vector<std::vector<char>> names(len);
 	std::vector<char*> ptr_names(len);
@@ -499,12 +572,14 @@ SEXP LGBM_BoosterGetEvalNames_R(SEXP handle) {
 
 	int out_len;
 	size_t required_string_size;
+	R_API_BEGIN();
 	CHECK_CALL(
 		LGBM_BoosterGetEvalNames(
 			R_ExternalPtrAddr(handle),
 			len, &out_len,
 			reserved_string_size, &required_string_size,
 			ptr_names.data()));
+	R_API_END();
 	// if any eval names were larger than allocated size,
 	// allow for a larger size and try again
 	if (required_string_size > reserved_string_size) {
@@ -512,6 +587,7 @@ SEXP LGBM_BoosterGetEvalNames_R(SEXP handle) {
 			names[i].resize(required_string_size);
 			ptr_names[i] = names[i].data();
 		}
+		R_API_BEGIN();
 		CHECK_CALL(
 			LGBM_BoosterGetEvalNames(
 				R_ExternalPtrAddr(handle),
@@ -520,6 +596,7 @@ SEXP LGBM_BoosterGetEvalNames_R(SEXP handle) {
 				required_string_size,
 				&required_string_size,
 				ptr_names.data()));
+		R_API_END();
 	}
 	CHECK_EQ(out_len, len);
 	eval_names = PROTECT(Rf_allocVector(STRSXP, len));
@@ -528,7 +605,6 @@ SEXP LGBM_BoosterGetEvalNames_R(SEXP handle) {
 	}
 	UNPROTECT(1);
 	return eval_names;
-	R_API_END();
 }
 
 SEXP LGBM_BoosterGetEval_R(SEXP handle,
@@ -542,6 +618,7 @@ SEXP LGBM_BoosterGetEval_R(SEXP handle,
 	CHECK_CALL(LGBM_BoosterGetEval(R_ExternalPtrAddr(handle), Rf_asInteger(data_idx), &out_len, ptr_ret));
 	CHECK_EQ(out_len, len);
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterGetNumPredict_R(SEXP handle,
@@ -552,6 +629,7 @@ SEXP LGBM_BoosterGetNumPredict_R(SEXP handle,
 	CHECK_CALL(LGBM_BoosterGetNumPredict(R_ExternalPtrAddr(handle), Rf_asInteger(data_idx), &len));
 	INTEGER(out)[0] = static_cast<int>(len);
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterGetPredict_R(SEXP handle,
@@ -562,6 +640,7 @@ SEXP LGBM_BoosterGetPredict_R(SEXP handle,
 	int64_t out_len;
 	CHECK_CALL(LGBM_BoosterGetPredict(R_ExternalPtrAddr(handle), Rf_asInteger(data_idx), &out_len, ptr_ret));
 	R_API_END();
+	return R_NilValue;
 }
 
 int GetPredictType(SEXP is_rawscore, SEXP is_leafidx, SEXP is_predcontrib) {
@@ -588,12 +667,17 @@ SEXP LGBM_BoosterPredictForFile_R(SEXP handle,
 	SEXP num_iteration,
 	SEXP parameter,
 	SEXP result_filename) {
-	R_API_BEGIN();
+	const char* data_filename_ptr = CHAR(PROTECT(Rf_asChar(data_filename)));
+	const char* parameter_ptr = CHAR(PROTECT(Rf_asChar(parameter)));
+	const char* result_filename_ptr = CHAR(PROTECT(Rf_asChar(result_filename)));
 	int pred_type = GetPredictType(is_rawscore, is_leafidx, is_predcontrib);
-	CHECK_CALL(LGBM_BoosterPredictForFile(R_ExternalPtrAddr(handle), CHAR(Rf_asChar(data_filename)),
-		Rf_asInteger(data_has_header), pred_type, Rf_asInteger(start_iteration), Rf_asInteger(num_iteration), CHAR(Rf_asChar(parameter)),
-		CHAR(Rf_asChar(result_filename))));
+	R_API_BEGIN();
+	CHECK_CALL(LGBM_BoosterPredictForFile(R_ExternalPtrAddr(handle), data_filename_ptr,
+		Rf_asInteger(data_has_header), pred_type, Rf_asInteger(start_iteration), Rf_asInteger(num_iteration), parameter_ptr,
+		result_filename_ptr));
 	R_API_END();
+	UNPROTECT(3);
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterCalcNumPredict_R(SEXP handle,
@@ -611,6 +695,7 @@ SEXP LGBM_BoosterCalcNumPredict_R(SEXP handle,
 		pred_type, Rf_asInteger(start_iteration), Rf_asInteger(num_iteration), &len));
 	INTEGER(out_len)[0] = static_cast<int>(len);
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterPredictForCSC_R(SEXP handle,
@@ -627,23 +712,24 @@ SEXP LGBM_BoosterPredictForCSC_R(SEXP handle,
 	SEXP num_iteration,
 	SEXP parameter,
 	SEXP out_result) {
-	R_API_BEGIN();
 	int pred_type = GetPredictType(is_rawscore, is_leafidx, is_predcontrib);
-
 	const int* p_indptr = INTEGER(indptr);
 	const int32_t* p_indices = reinterpret_cast<const int32_t*>(INTEGER(indices));
 	const double* p_data = REAL(data);
-
 	int64_t nindptr = static_cast<int64_t>(Rf_asInteger(num_indptr));
 	int64_t ndata = static_cast<int64_t>(Rf_asInteger(nelem));
 	int64_t nrow = static_cast<int64_t>(Rf_asInteger(num_row));
 	double* ptr_ret = REAL(out_result);
 	int64_t out_len;
+	const char* parameter_ptr = CHAR(PROTECT(Rf_asChar(parameter)));
+	R_API_BEGIN();
 	CHECK_CALL(LGBM_BoosterPredictForCSC(R_ExternalPtrAddr(handle),
 		p_indptr, C_API_DTYPE_INT32, p_indices,
 		p_data, C_API_DTYPE_FLOAT64, nindptr, ndata,
-		nrow, pred_type, Rf_asInteger(start_iteration), Rf_asInteger(num_iteration), CHAR(Rf_asChar(parameter)), &out_len, ptr_ret));
+		nrow, pred_type, Rf_asInteger(start_iteration), Rf_asInteger(num_iteration), parameter_ptr, &out_len, ptr_ret));
 	R_API_END();
+	UNPROTECT(1);
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterPredictForMat_R(SEXP handle,
@@ -657,29 +743,32 @@ SEXP LGBM_BoosterPredictForMat_R(SEXP handle,
 	SEXP num_iteration,
 	SEXP parameter,
 	SEXP out_result) {
-	R_API_BEGIN();
 	int pred_type = GetPredictType(is_rawscore, is_leafidx, is_predcontrib);
-
 	int32_t nrow = static_cast<int32_t>(Rf_asInteger(num_row));
 	int32_t ncol = static_cast<int32_t>(Rf_asInteger(num_col));
-
 	const double* p_mat = REAL(data);
 	double* ptr_ret = REAL(out_result);
+	const char* parameter_ptr = CHAR(PROTECT(Rf_asChar(parameter)));
 	int64_t out_len;
+	R_API_BEGIN();
 	CHECK_CALL(LGBM_BoosterPredictForMat(R_ExternalPtrAddr(handle),
 		p_mat, C_API_DTYPE_FLOAT64, nrow, ncol, COL_MAJOR,
-		pred_type, Rf_asInteger(start_iteration), Rf_asInteger(num_iteration), CHAR(Rf_asChar(parameter)), &out_len, ptr_ret));
-
+		pred_type, Rf_asInteger(start_iteration), Rf_asInteger(num_iteration), parameter_ptr, &out_len, ptr_ret));
 	R_API_END();
+	UNPROTECT(1);
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterSaveModel_R(SEXP handle,
 	SEXP num_iteration,
 	SEXP feature_importance_type,
 	SEXP filename) {
+	const char* filename_ptr = CHAR(PROTECT(Rf_asChar(filename)));
 	R_API_BEGIN();
-	CHECK_CALL(LGBM_BoosterSaveModel(R_ExternalPtrAddr(handle), 0, Rf_asInteger(num_iteration), Rf_asInteger(feature_importance_type), CHAR(Rf_asChar(filename))));
+	CHECK_CALL(LGBM_BoosterSaveModel(R_ExternalPtrAddr(handle), 0, Rf_asInteger(num_iteration), Rf_asInteger(feature_importance_type), filename_ptr));
 	R_API_END();
+	UNPROTECT(1);
+	return R_NilValue;
 }
 
 SEXP LGBM_BoosterSaveModelToString_R(SEXP handle,
@@ -687,51 +776,59 @@ SEXP LGBM_BoosterSaveModelToString_R(SEXP handle,
 	SEXP num_iteration,
 	SEXP feature_importance_type) {
 	SEXP model_str;
-	R_API_BEGIN();
 	int64_t out_len = 0;
 	int64_t buf_len = 1024 * 1024;
 	int start_iter = Rf_asInteger(start_iteration);
 	int num_iter = Rf_asInteger(num_iteration);
 	int importance_type = Rf_asInteger(feature_importance_type);
 	std::vector<char> inner_char_buf(buf_len);
+	R_API_BEGIN();
 	CHECK_CALL(LGBM_BoosterSaveModelToString(R_ExternalPtrAddr(handle), start_iter, num_iter, importance_type, buf_len, &out_len, inner_char_buf.data()));
+	R_API_END();
 	// if the model string was larger than the initial buffer, allocate a bigger buffer and try again
 	if (out_len > buf_len) {
 		inner_char_buf.resize(out_len);
+		R_API_BEGIN();
 		CHECK_CALL(LGBM_BoosterSaveModelToString(R_ExternalPtrAddr(handle), start_iter, num_iter, importance_type, out_len, &out_len, inner_char_buf.data()));
+		R_API_END();
 	}
 	model_str = PROTECT(Rf_allocVector(STRSXP, 1));
 	SET_STRING_ELT(model_str, 0, Rf_mkChar(inner_char_buf.data()));
 	UNPROTECT(1);
 	return model_str;
-	R_API_END();
 }
 
 SEXP LGBM_BoosterDumpModel_R(SEXP handle,
 	SEXP num_iteration,
 	SEXP feature_importance_type) {
 	SEXP model_str;
-	R_API_BEGIN();
 	int64_t out_len = 0;
 	int64_t buf_len = 1024 * 1024;
 	int num_iter = Rf_asInteger(num_iteration);
 	int importance_type = Rf_asInteger(feature_importance_type);
 	std::vector<char> inner_char_buf(buf_len);
+	R_API_BEGIN();
 	CHECK_CALL(LGBM_BoosterDumpModel(R_ExternalPtrAddr(handle), 0, num_iter, importance_type, buf_len, &out_len, inner_char_buf.data()));
+	R_API_END();
 	// if the model string was larger than the initial buffer, allocate a bigger buffer and try again
 	if (out_len > buf_len) {
 		inner_char_buf.resize(out_len);
+		R_API_BEGIN();
 		CHECK_CALL(LGBM_BoosterDumpModel(R_ExternalPtrAddr(handle), 0, num_iter, importance_type, out_len, &out_len, inner_char_buf.data()));
+		R_API_END();
 	}
 	model_str = PROTECT(Rf_allocVector(STRSXP, 1));
 	SET_STRING_ELT(model_str, 0, Rf_mkChar(inner_char_buf.data()));
 	UNPROTECT(1);
 	return model_str;
-	R_API_END();
 }
 
 // Below here are REModel / GPModel related functions
 
+void _REModelFinalizer(SEXP handle) {
+	GPB_REModelFree_R(handle);
+}
+
 SEXP GPB_CreateREModel_R(SEXP ndata,
 	SEXP cluster_ids_data,
 	SEXP re_group_data,
@@ -753,9 +850,19 @@ SEXP GPB_CreateREModel_R(SEXP ndata,
 	SEXP vecchia_pred_type,
 	SEXP num_neighbors_pred,
 	SEXP likelihood) {
+	int n_protected = 0;
 	SEXP ret;
-	R_API_BEGIN();
 	REModelHandle handle = nullptr;
+	SEXP cov_fct_aux = PROTECT(Rf_asChar(cov_fct));
+	SEXP vecchia_ordering_aux = PROTECT(Rf_asChar(vecchia_ordering));
+	SEXP vecchia_pred_type_aux = PROTECT(Rf_asChar(vecchia_pred_type));
+	SEXP likelihood_aux = PROTECT(Rf_asChar(likelihood));
+	n_protected = n_protected + 4;
+	const char* cov_fct_ptr = (Rf_isNull(cov_fct)) ? nullptr : CHAR(cov_fct_aux);
+	const char* vecchia_ordering_ptr = (Rf_isNull(vecchia_ordering)) ? nullptr : CHAR(vecchia_ordering_aux);
+	const char* vecchia_pred_type_ptr = (Rf_isNull(vecchia_pred_type)) ? nullptr : CHAR(vecchia_pred_type_aux);
+	const char* likelihood_ptr = (Rf_isNull(likelihood)) ? nullptr : CHAR(likelihood_aux);
+	R_API_BEGIN();
 	CHECK_CALL(GPB_CreateREModel(Rf_asInteger(ndata),
 		R_INT_PTR(cluster_ids_data),
 		R_CHAR_PTR_FROM_RAW(re_group_data),
@@ -768,20 +875,22 @@ SEXP GPB_CreateREModel_R(SEXP ndata,
 		Rf_asInteger(dim_gp_coords),
 		R_REAL_PTR(gp_rand_coef_data),
 		Rf_asInteger(num_gp_rand_coef),
-		R_CHAR_PTR(cov_fct),
+		cov_fct_ptr,
 		Rf_asReal(cov_fct_shape),
 		Rf_asReal(cov_fct_taper_range),
 		Rf_asLogical(vecchia_approx),
 		Rf_asInteger(num_neighbors),
-		R_CHAR_PTR(vecchia_ordering),
-		R_CHAR_PTR(vecchia_pred_type),
+		vecchia_ordering_ptr,
+		vecchia_pred_type_ptr,
 		Rf_asInteger(num_neighbors_pred),
-		R_CHAR_PTR(likelihood),
+		likelihood_ptr,
 		&handle));
-	ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
-	UNPROTECT(1);
-	return ret;
 	R_API_END();
+	ret = PROTECT(R_MakeExternalPtr(handle, R_NilValue, R_NilValue));
+	n_protected++;
+	R_RegisterCFinalizerEx(ret, _REModelFinalizer, TRUE);
+	UNPROTECT(n_protected);
+	return ret;
 }
 
 SEXP GPB_REModelFree_R(SEXP handle) {
@@ -791,6 +900,7 @@ SEXP GPB_REModelFree_R(SEXP handle) {
 		R_ClearExternalPtr(handle);
 	}
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP GPB_SetOptimConfig_R(SEXP handle,
@@ -806,6 +916,12 @@ SEXP GPB_SetOptimConfig_R(SEXP handle,
 	SEXP momentum_offset,
 	SEXP convergence_criterion,
 	SEXP calc_std_dev) {
+	int n_protected = 0;
+	SEXP optimizer_aux = PROTECT(Rf_asChar(optimizer));
+	SEXP convergence_criterion_aux = PROTECT(Rf_asChar(convergence_criterion));
+	n_protected = n_protected + 2;
+	const char* optimizer_ptr = (Rf_isNull(optimizer)) ? nullptr : CHAR(optimizer_aux);
+	const char* convergence_criterion_ptr = (Rf_isNull(convergence_criterion)) ? nullptr : CHAR(convergence_criterion_aux);
 	R_API_BEGIN();
 	CHECK_CALL(GPB_SetOptimConfig(R_ExternalPtrAddr(handle),
 		R_REAL_PTR(init_cov_pars),
@@ -816,11 +932,13 @@ SEXP GPB_SetOptimConfig_R(SEXP handle,
 		Rf_asLogical(use_nesterov_acc),
 		Rf_asInteger(nesterov_schedule_version),
 		Rf_asLogical(trace),
-		R_CHAR_PTR(optimizer),
+		optimizer_ptr,
 		Rf_asInteger(momentum_offset),
-		R_CHAR_PTR(convergence_criterion),
+		convergence_criterion_ptr,
 		Rf_asLogical(calc_std_dev)));
 	R_API_END();
+	UNPROTECT(n_protected);
+	return R_NilValue;
 }
 
 SEXP GPB_SetOptimCoefConfig_R(SEXP handle,
@@ -829,14 +947,20 @@ SEXP GPB_SetOptimCoefConfig_R(SEXP handle,
 	SEXP lr_coef,
 	SEXP acc_rate_coef,
 	SEXP optimizer) {
+	int n_protected = 0;
+	SEXP optimizer_aux = PROTECT(Rf_asChar(optimizer));
+	n_protected++;
+	const char* optimizer_ptr = (Rf_isNull(optimizer)) ? nullptr : CHAR(optimizer_aux);
 	R_API_BEGIN();
 	CHECK_CALL(GPB_SetOptimCoefConfig(R_ExternalPtrAddr(handle),
 		Rf_asInteger(num_covariates),
 		R_REAL_PTR(init_coef),
 		Rf_asReal(lr_coef),
 		Rf_asReal(acc_rate_coef),
-		R_CHAR_PTR(optimizer)));
+		optimizer_ptr));
 	R_API_END();
+	UNPROTECT(n_protected);
+	return R_NilValue;
 }
 
 SEXP GPB_OptimCovPar_R(SEXP handle,
@@ -847,6 +971,7 @@ SEXP GPB_OptimCovPar_R(SEXP handle,
 		R_REAL_PTR(y_data),
 		R_REAL_PTR(fixed_effects)));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP GPB_OptimLinRegrCoefCovPar_R(SEXP handle,
@@ -859,6 +984,7 @@ SEXP GPB_OptimLinRegrCoefCovPar_R(SEXP handle,
 		R_REAL_PTR(covariate_data),
 		Rf_asInteger(num_covariates)));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP GPB_EvalNegLogLikelihood_R(SEXP handle,
@@ -871,6 +997,7 @@ SEXP GPB_EvalNegLogLikelihood_R(SEXP handle,
 		R_REAL_PTR(cov_pars),
 		R_REAL_PTR(negll)));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP GPB_GetCovPar_R(SEXP handle,
@@ -881,6 +1008,7 @@ SEXP GPB_GetCovPar_R(SEXP handle,
 		R_REAL_PTR(optim_cov_pars),
 		Rf_asLogical(calc_std_dev)));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP GPB_GetInitCovPar_R(SEXP handle,
@@ -889,6 +1017,7 @@ SEXP GPB_GetInitCovPar_R(SEXP handle,
 	CHECK_CALL(GPB_GetInitCovPar(R_ExternalPtrAddr(handle),
 		R_REAL_PTR(init_cov_pars)));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP GPB_GetCoef_R(SEXP handle,
@@ -899,6 +1028,7 @@ SEXP GPB_GetCoef_R(SEXP handle,
 		R_REAL_PTR(optim_coef),
 		Rf_asLogical(calc_std_dev)));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP GPB_GetNumIt_R(SEXP handle,
@@ -907,6 +1037,7 @@ SEXP GPB_GetNumIt_R(SEXP handle,
 	CHECK_CALL(GPB_GetNumIt(R_ExternalPtrAddr(handle),
 		R_INT_PTR(num_it)));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP GPB_SetPredictionData_R(SEXP handle,
@@ -927,6 +1058,7 @@ SEXP GPB_SetPredictionData_R(SEXP handle,
 		R_REAL_PTR(gp_rand_coef_data_pred),
 		R_REAL_PTR(covariate_data_pred)));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP GPB_PredictREModel_R(SEXP handle,
@@ -948,6 +1080,10 @@ SEXP GPB_PredictREModel_R(SEXP handle,
 	SEXP fixed_effects,
 	SEXP fixed_effects_pred,
 	SEXP out_predict) {
+	int n_protected = 0;
+	SEXP vecchia_pred_type_aux = PROTECT(Rf_asChar(vecchia_pred_type));
+	n_protected++;
+	const char* vecchia_pred_type_ptr = (Rf_isNull(vecchia_pred_type)) ? nullptr : CHAR(vecchia_pred_type_aux);
 	R_API_BEGIN();
 	CHECK_CALL(GPB_PredictREModel(R_ExternalPtrAddr(handle),
 		R_REAL_PTR(y_data),
@@ -964,64 +1100,72 @@ SEXP GPB_PredictREModel_R(SEXP handle,
 		R_REAL_PTR(cov_pars),
 		R_REAL_PTR(covariate_data_pred),
 		Rf_asLogical(use_saved_data),
-		R_CHAR_PTR(vecchia_pred_type),
+		vecchia_pred_type_ptr,
 		Rf_asInteger(num_neighbors_pred),
 		R_REAL_PTR(fixed_effects),
 		R_REAL_PTR(fixed_effects_pred)));
 	R_API_END();
+	UNPROTECT(n_protected);
+	return R_NilValue;
 }
 
 SEXP GPB_GetLikelihoodName_R(SEXP handle) {
-	SEXP ll_name;
-	R_API_BEGIN();
+	SEXP ret;
 	std::vector<char> inner_char_buf(128);
 	int num_char;
+	R_API_BEGIN();
 	CHECK_CALL(GPB_GetLikelihoodName(R_ExternalPtrAddr(handle),
 		inner_char_buf.data(),
 		num_char));
-	ll_name = PROTECT(Rf_allocVector(STRSXP, 1));
-	SET_STRING_ELT(ll_name, 0, Rf_mkChar(inner_char_buf.data()));
-	UNPROTECT(1);
-	return ll_name;
 	R_API_END();
+	ret = PROTECT(Rf_allocVector(STRSXP, 1));
+	SET_STRING_ELT(ret, 0, Rf_mkChar(inner_char_buf.data()));
+	UNPROTECT(1);
+	return ret;
 }
 
 SEXP GPB_GetOptimizerCovPars_R(SEXP handle) {
-	SEXP opt_name;
-	R_API_BEGIN();
+	SEXP ret;
 	std::vector<char> inner_char_buf(128);
 	int num_char;
+	R_API_BEGIN();
 	CHECK_CALL(GPB_GetOptimizerCovPars(R_ExternalPtrAddr(handle),
 		inner_char_buf.data(),
 		num_char));
-	opt_name = PROTECT(Rf_allocVector(STRSXP, 1));
-	SET_STRING_ELT(opt_name, 0, Rf_mkChar(inner_char_buf.data()));
-	UNPROTECT(1);
-	return opt_name;
 	R_API_END();
+	ret = PROTECT(Rf_allocVector(STRSXP, 1));
+	SET_STRING_ELT(ret, 0, Rf_mkChar(inner_char_buf.data()));
+	UNPROTECT(1);
+	return ret;
 }
 
 SEXP GPB_GetOptimizerCoef_R(SEXP handle) {
-	SEXP opt_name;
-	R_API_BEGIN();
+	SEXP ret;
 	std::vector<char> inner_char_buf(128);
 	int num_char;
+	R_API_BEGIN();
 	CHECK_CALL(GPB_GetOptimizerCoef(R_ExternalPtrAddr(handle),
 		inner_char_buf.data(),
 		num_char));
-	opt_name = PROTECT(Rf_allocVector(STRSXP, 1));
-	SET_STRING_ELT(opt_name, 0, Rf_mkChar(inner_char_buf.data()));
-	UNPROTECT(1);
-	return opt_name;
 	R_API_END();
+	ret = PROTECT(Rf_allocVector(STRSXP, 1));
+	SET_STRING_ELT(ret, 0, Rf_mkChar(inner_char_buf.data()));
+	UNPROTECT(1);
+	return ret;
 }
 
 SEXP GPB_SetLikelihood_R(SEXP handle,
 	SEXP likelihood) {
+	int n_protected = 0;
+	SEXP likelihood_aux = PROTECT(Rf_asChar(likelihood));
+	n_protected++;
+	const char* likelihood_ptr = (Rf_isNull(likelihood)) ? nullptr : CHAR(likelihood_aux);
 	R_API_BEGIN();
 	CHECK_CALL(GPB_SetLikelihood(R_ExternalPtrAddr(handle),
-		R_CHAR_PTR(likelihood)));
+		likelihood_ptr));
 	R_API_END();
+	UNPROTECT(n_protected);
+	return R_NilValue;
 }
 
 SEXP GPB_GetResponseData_R(SEXP handle,
@@ -1030,6 +1174,7 @@ SEXP GPB_GetResponseData_R(SEXP handle,
 	CHECK_CALL(GPB_GetResponseData(R_ExternalPtrAddr(handle),
 		R_REAL_PTR(response_data)));
 	R_API_END();
+	return R_NilValue;
 }
 
 SEXP GPB_GetCovariateData_R(SEXP handle,
@@ -1038,11 +1183,11 @@ SEXP GPB_GetCovariateData_R(SEXP handle,
 	CHECK_CALL(GPB_GetCovariateData(R_ExternalPtrAddr(handle),
 		R_REAL_PTR(covariate_data)));
 	R_API_END();
+	return R_NilValue;
 }
 
 // .Call() calls
 static const R_CallMethodDef CallEntries[] = {
-  {"LGBM_GetLastError_R"              , (DL_FUNC)&LGBM_GetLastError_R              , 0},
   {"LGBM_HandleIsNull_R"              , (DL_FUNC)&LGBM_HandleIsNull_R              , 1},
   {"LGBM_DatasetCreateFromFile_R"     , (DL_FUNC)&LGBM_DatasetCreateFromFile_R     , 3},
   {"LGBM_DatasetCreateFromCSC_R"      , (DL_FUNC)&LGBM_DatasetCreateFromCSC_R      , 8},
